--[[
  Auctioneer Advanced - Search UI - Searcher Pawn
  Version: 1.1.4 (Xit)
  Revision: $Id: SearcherPawn.lua 1.1.4 20090830 Xit $
  URL: http://wow.curse.com/downloads/wow-addons/details/auc-advanced-searcher-pawn.aspx

  This is a plugin module for the SearchUI that assists in searching by evaluating items with Pawn

  License:
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program(see GPL.txt); if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Note:
    This AddOn's source code is specifically designed to work with
    World of Warcraft's interpreted AddOn system.
    You have an implicit license to use this AddOn with these facilities
    since that is its designated purpose as per:
    http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]
-- Create a new instance of our lib with our parent
local lib, parent, private = AucSearchUI.NewSearcher("Pawn")
if not lib then
  return
end
local print,decode,_,_,replicate,empty,_,_,_,debugPrint,fill = AucAdvanced.GetModuleLocals()
local get,set,default,Const = AucSearchUI.GetSearchLocals()

lib.tabname = "Pawn"

-- Set our defaults
default("search.pawn.scalename", "")
default("search.pawn.enchant", false)
default("search.pawn.canuse", false)
default("search.pawn.maxvalue", false)
default("search.pawn.forcemh", false)
default("search.pawn.debug", false)

local scaletable = {}
local pclass = UnitClass("player")
local plevel = UnitLevel("player")

local convertSlot = {
  {
    1, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [1] head
  {
    2, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [2] neck
  {
    3, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [3] shoulder
  {
    4, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [4] body
  {
    5, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [5] chest
  {
    6, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [6] waist
  {
    7, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [7] legs
  {
    8, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [8] feet
  {
    9, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [9] wrist
  {
    10, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [10] hand
  {
    11, -- [1] mainslot
    12, -- [2] otherslot
  }, -- [11] finger
  {
    13, -- [1] mainslot
    14, -- [2] otherslot
  }, -- [12] trinket
  {
    16, -- [1] mainslot
    17, -- [2] otherslot
  }, -- [13] weapon
  {
    17, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [14] shield
  {
    18, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [15] rangedright
  {
    15, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [16] cloak
  {
    16, -- [1] mainslot
    17, -- [2] otherslot
  }, -- [17] 2hweapon
  {
    -1, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [18] bag
  {
    -1, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [19] tabard
  {
    5, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [20] robe
  {
    16, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [21] weaponmainhand
  {
    17, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [22] weaponoffhand
  {
    17, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [23] holdable
  {
    0, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [24] ammo
  {
    18, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [25] thrown
  {
    18, -- [1] mainslot
    -1, -- [2] otherslot
  }, -- [26] ranged
}

local function Debug(Reason, Text)
  local showdbg = get("search.pawn.debug")
  local showfalse = get("search.pawn.debugfalse")
  local showtrue = get("search.pawn.debugtrue")

  if not showdbg then
    return
  end

  if not showfalse and Reason == false then
    return
  end

  if not showtrue and Reason == true then
    return
  end

  print(string.format("%s", Text))
end


local function getScales()
  local scalecopy = {}
  table.insert(scalecopy, { 1, "" })
  for i,j in pairs(PawnScaleTable) do
    table.insert(scalecopy, { i+1, j })
  end
  return scalecopy
end

local function updateScales()
  print(string.format(" * %s", "Updating Pawn Scales"))

  local scalecopy = {}
  scaletable = PawnGetAllScales()
  table.insert(scalecopy, { 1, "" })
  for i,j in pairs(scaletable) do
    table.insert(scalecopy, { i+1, j })
  end

  PawnScaleTable = scalecopy

  return "ok"
end

-- This function is automatically called when we need to create our search generals
function lib:MakeGuiConfig(gui)
  if (PawnScaleTable == nil) then
    updateScales()
  end
  -- Get our tab and populate it with our controls
  local id = gui:AddTab(lib.tabname, "Searchers")

  -- Add the help
  gui:AddSearcher("Pawn", "Search for items which Pawn considers an upgrade", 100)
  gui:AddHelp(id, "pawn searcher",
    "What does this searcher do?",
    "This searcher allows you to choose a scale that has been defined in the Pawn addon.  This searcher will then use Pawn to calculate values for items in the auction scan and only show items which are an upgrade for you.")

  gui:AddControl(id, "Header", 0, "Pawn Search Criteria")
  local last = gui:GetLast(id)
  gui:AddControl(id, "Selectbox",  0, 1, PawnScaleTable, "search.pawn.scalename", "Pawn scale name")
  gui:AddTip(id, "The Pawn scale that will be used to determine the item value")
  gui:AddControl(id, "Button", 0,1, updateScales(), "Update")
  gui:AddTip(id, "Logout / Reload UI is still required at this time.")
  local cont = gui:GetLast(id)
  last = cont
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.enchant", "Check enchanted value")
  gui:AddTip(id, "If checked, items will be calculated 'as is'.  Empty sockets will be ignored.  If unchecked, items will be calculated without any enchantments and all empty sockets will use the value you set in the Pawn scale.")
  last = gui:GetLast(id)
  gui:SetLast(id, last)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.canuse", "Useable items only")
  gui:AddTip(id, "Only items that your character can use.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.maxvalue", "Use bigger value")
  gui:AddTip(id, "Use bigger of the values (enchanted / unenchanted) returned by Pawn for equipped items.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.forcemh", "Force main hand")
  gui:AddTip(id, "Off hand items will only be compared to your main hand slot.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.impoor", "Only list what I can afford to buy")
  gui:AddTip(id, "Limit to only what you can currently afford to buy.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.buyout", "Use buyout")
  gui:AddTip(id, "Use buyout instead of bid when checking auction prices.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.25, 0, "search.pawn.debug", "Show debug messages")
  gui:AddTip(id, "WARNING: Produces lots of output.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.28, 0, "search.pawn.debugfalse", "Only false")
  gui:AddTip(id, "Only show debug messages for items that do not match.")
  last = gui:GetLast(id)
  gui:AddControl(id, "Checkbox",   0.28, 0, "search.pawn.debugtrue", "Only true")
  gui:AddTip(id, "Only show debug messages for items that match.")
end

local function RGBPercToHex(r, g, b)
  r = r <= 1 and r >= 0 and r or 0
  g = g <= 1 and g >= 0 and g or 0
  b = b <= 1 and b >= 0 and b or 0
  return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local function CanUse(link)

  MyScanningTooltip:ClearLines()
  MyScanningTooltip:SetHyperlink(link)
  local retval = true

  for i=1,MyScanningTooltip:NumLines() do
    --Check left side text to see if it is red, if so, can't use
    local mytext = getglobal("MyScanningTooltipTextLeft" .. i)
    local text = mytext:GetText()
    if (text) then
      local r,g,b,a = mytext:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end

    --Check right side text to see if it is red, if so, can't use
    local mytextr = getglobal("MyScanningTooltipTextRight" .. i)
    local textr = mytextr:GetText()
    if (textr) then
      local r,g,b,a = mytextr:GetTextColor()
      local hex = RGBPercToHex(r,g,b)
      if (hex == "fe1f1f") then
        retval = false
      end
    end
  end

  return retval
end

-- Convert between auctioneer equip positions and Pawn equip positions
local function ConvertSlot(pos,itype,subtype)
  local mainslot, otherslot

  local candual = 0
  local hasammo = 0
  local forcemh = get("search.pawn.forcemh")

  -- Limit one-handed weapon checks to main hand unless class can dual wield
  if (pclass == "Warrior" or pclass == "Hunter" and plevel >= 20) then
    candual = 1
  end
  if (pclass == "Shaman" and plevel >= 40) then
    candual = 1
  end
  if (pclass == "Rogue" and plevel >= 10) then
    candual = 1
  end
  if (pclass == "Death Knight") then
    candual = 1
  end

  local slots = convertSlot[pos]
  mainslot = slots[1]
  otherslot = slots[2]
  if otherslot == -1 then
    otherslot = nil
  end

  if pos == 13 then -- One Hand Weapon
    if candual and not forcemh then
      otherslot = 17
    end
  end

  if pos == 22 then -- Off hand
    if forcemh then
      mainslot = 16
    end
  end

  if pos == 23 then -- Holdable
    if forcemh then
      mainslot = 16
    end
  end

  if (itype == "Armor") and ((subtype == "Totems") or (subtype == "Librams") or (subtype == "Idols") or (subtype == "Sigils")) then -- Relics
    mainslot = 18
  end

  return mainslot, otherslot
end

function lib.Search(itemData)
  local link = itemData[Const.LINK]
  local itype = itemData[Const.ITYPE]
  local ipos  = itemData[Const.IEQUIP]
  local iname = itemData[Const.NAME]
  local subtype = itemData[Const.ISUB]
  local bid = itemData[Const.MINBID]
  local buyout = itemData[Const.BUYOUT]
  local debugfalse = get("search.pawn.debugfalse")
  local debugtrue = get("search.pawn.debugtrue")
  local pmoney = GetMoney()

  -- Must have ipos
  if ipos == nil then return end

  -- Set name to blank for nil names.  This is only used for debugging.
  if not iname then iname = " " end

  -- Don't let bags through
  if ipos == 18 then return end

  -- Realtime search support
  if type(ipos) == "string" then
    ipos = Const.InvTypes[ipos]
  end

  -- Did the user pick a scale
  local scalenum = get("search.pawn.scalename")
  local scalename = ""

  if not scalenum then
    Debug(false, "DBG: No scale defined, please pick a scale from the dropdown list.  If none exist you will need to define a scale in Pawn first.")
    return false, "No scale name"
  end

  -- Does the stored scale exist
  for i, j in pairs(scaletable) do
    if i == (scalenum - 1) then -- subtract 1, due to offset introduced in menu
      scalename = j
      break
    end
  end

  if scalename == "" then
    Debug(false, "DBG: No scale defined, please pick a scale from the dropdown list.  If none exist you will need to define a scale in Pawn first.")
    return false, "Scale not found or blank"
  end

  local scalefound = PawnDoesScaleExist(scalename)
  if not scalefound or scalefound ~= true then
    Debug(false, ("DBG: Error! Pawn cannot find scale [%s]"):format(scalename))
    return false, "Scale not found"
  end

  -- Does the item have a link
  if not link then
    Debug(false, "No Link")
    return false, "No link"
  end

  -- Don't show Ammo/thrown for non ammo classes
  if ((pclass ~= "Hunter" and pclass ~= "Rogue" and pclass ~= "Warrior") and (ipos == 24 or ipos == 25)) then
    Debug(false, ("DBG:%s Can't Use"):format(iname))
    return false, "Can't Use"
  end

  -- Only filter stuff that can be equipped
  if (not IsEquippableItem(link)) then
    Debug(false, ("DBG:%s Not Equippable"):format(iname))
    return false, "Not Equippable"
  end

  local maxvalue = get("search.pawn.maxvalue")

  -- Check to see if we can use the item
  local canuse = get("search.pawn.canuse")
  local icanuse = CanUse(link)

  if canuse and icanuse ~= true then
    Debug(false, ("DBG:%s Can't use"):format(iname))
    return false, "Can't use"
  end

  -- Check to see if we can afford the item
  local impoor = get("search.pawn.impoor")
  local usebuyout = get("search.pawn.buyout")

  if impoor then
    if usebuyout and pmoney < buyout then
      Debug(false, ("DBG:%s Can't afford ahPrice[%d] pMoney[%d]"):format(iname, buyout, pmoney))
      return false, "Can't afford buyout"
    end
    if not usebuyout and pmoney < bid then
      Debug(false, ("DBG:%s Can't afford ahPrice[%d] pMoney[%d]"):format(iname, bid, pmoney))
      return false, "Can't afford bid"
    end
  end

  local reason = "buy"
  if (not usebuyout) then
    reason = "bid"
  end

  -----------------------------------------------------------------
  --Ok At this point we can use the item and possibly can afford it
  -----------------------------------------------------------------

  -- Get the signature of this item and find it's stats.
  local item = PawnGetItemData(link)

  if not item then
    Debug(false, ("DBG:%s Pawn has no data for this item."):format(iname))
    return false, "No Item"
  end

  local aValE, aVal = PawnGetSingleValueFromItem(item, scalename)

  if aVal == nil then
    aVal = 0
  end
  if aValE == nil then
    aValE = 0
  end

  local aStr = ("%07.2f"):format(aVal)
  local aStrE = ("(E)%07.2f"):format(aValE)

  -- Get value of equivalent slot(s).
  local mainslot, otherslot = ConvertSlot(ipos,itype,subtype)
  local mItem = PawnGetItemDataForInventorySlot(mainslot)

  -- If there is no item equipped in that slot, then any item is an upgrade
  if not mItem then
    Debug(true, ("DBG:(%d)%s aVal[%07.2f] > Empty Slot"):format(ipos,iname,aVal))
    --return aStr,nil,nil
    return reason,nil,aVal,aStr
  end

  -- Grab the values for the item equipped in the main slot
  local mValE, mVal = PawnGetSingleValueFromItem(mItem, scalename)
  local oItem = nil
  local oVal = 0
  local oValE = 0

  if (otherslot) then
    oItem = PawnGetItemDataForInventorySlot(otherslot)
    if oItem then
      oValE, oVal = PawnGetSingleValueFromItem(oItem, scalename)
    end
  end

  -- ensure we have valid values for comparisons

  if mVal == nil then
    mVal = 0
  end
  if mValE == nil then
    mValE = 0
  end
  if oVal == nil then
    oVal = 0
  end
  if oValE == nil then
    oValE = 0
  end

  local enchant = get("search.pawn.enchant")

  -- use the bigger of the two numbers for items that are equipped.
  -- This will assume that you intend to socket items with appropriate
  -- gems.  Pawn will fill in values for sockets that are empty when
  -- calculating the unenchanted value of the item.  If that value is
  -- bigger, use that.
  local mainValue = mValE
  local offValue = oValE

  if maxvalue then
    if (mVal > mValE) then
      mainValue = mVal
    end
    if (oVal > oValE) then
      offValue = oVal
    end
  end

  local slots = convertSlot[ipos]

  -- If enchant is checked, check it first.
  if (enchant) then
    -- compare 2-handed weapons against both main hand and off hand
    if ipos == 17 then
      local bValE = mainValue + offValue
      if aValE > bValE then
        local delta = aValE - bValE
        local dStr = ("%07.2f"):format(delta)
        Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
        Debug(true, ("DBG:(%d)%s aValE[%07.2f] > bValE[%07.2f] (mainValue[%07.2f] + offValue[%07.2f])"):format(ipos,iname,aValE,bValE,mainValue,offValue))
        return reason,nil,nil,dStr
      end
    else
      if aValE > mainValue then
        local delta = aValE - mainValue
        local dStr = ("%07.2f"):format(delta)
        Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
        Debug(true, ("DBG:(%d)%s aValE[%07.2f] > mainValue[%07.2f]"):format(ipos,iname,aValE,mainValue))
        return reason,nil,nil,dStr
      end
      if otherslot and aValE > offValue then
        local delta = aValE - offValue
        local dStr = ("%07.2f"):format(delta)
        Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
        Debug(true, ("DBG:(%d)%s aValE[%07.2f] > offValue[%07.2f]"):format(ipos,iname,aValE,offValue))
        return reason,nil,nil,dStr
      end
    end
  end

  -- compare 2-handed weapons against both main hand and off hand
  if ipos == 17 then
    local bVal = mainValue + offValue
    if aVal > bVal then
      local delta = aVal - bVal
      local dStr = ("%07.2f"):format(delta)
      local slots = convertSlot[ipos]
      Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
      Debug(true, ("DBG:(%d)%s aVal[%07.2f] > bVal[%07.2f] (mainValue[%07.2f] + offValue[%07.2f])"):format(ipos,iname,aVal,bVal,mainValue,offValue))
      return reason,nil,nil,dStr
    end
  else
    if aVal > mainValue then
      local delta = aVal - mainValue
      local dStr = ("%07.2f"):format(delta)
      local slots = convertSlot[ipos]
      Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
      Debug(true, ("DBG:(%d)%s aVal[%07.2f] > mainValue[%07.2f]"):format(ipos,iname,aVal,mainValue))
      return reason,nil,nil,dStr
    end
    if otherslot and aVal > offValue then
      local delta = aVal - offValue
      local dStr = ("%07.2f"):format(delta)
      local slots = convertSlot[ipos]
      Debug(true, string.format("DBG:iPos: %d mSlot: %d oSlot: %d", ipos, slots[1], slots[2]))
      Debug(true, ("DBG:(%d)%s aVal[%07.2f] > offValue[%07.2f]"):format(ipos,iname,aVal,offValue))
      return reason,nil,nil,dStr
    end
  end

  Debug(false, ("DBG:(%d)%s aVal[%07.2f] too low"):format(ipos,iname,aVal))
  return false, "Pawn Value "..aVal.." too low"
end
